/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2005  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef OUTPUTFUNCTION_H_
#define OUTPUTFUNCTION_H_

template<typename Arg>
class OutputFunction
{
public:
	virtual ~OutputFunction() {}
	
	virtual void operator()(Arg const& arg) = 0;
};


template<typename Arg, typename Func>
class TemplateOutputFunction : public OutputFunction<Arg>
{
public:
	TemplateOutputFunction(Func const& f) : m_func(f) {}
	
	virtual void operator()(Arg const& arg) {
		m_func(arg);
	}
private:
	Func m_func;
};


#define DECLARE_INSERTER(Class, Arg, Container, Action) \
class Class : public OutputFunction<Arg>  \
{ \
public: \
	Class(Container& c) : c(c) {} \
	virtual void operator()(Arg const& val) { Action; } \
private: \
	Container& c; \
};

#endif
